var http = require('http'),
EventEmitter = require('events').EventEmitter;

function urban(words) {
  return new Dictionary(words);
}

urban.version = "0.1.1";

function Dictionary(words) {
  EventEmitter.call(this);

  if (Array.isArray(words)) words = words.join(' ');

  this.words = words;
  this._ended = false;
  this.getData();

  return this;
}

Dictionary.fn = Dictionary.prototype;
Dictionary.fn.__proto__ = new EventEmitter;
Dictionary.fn._noop = function() {};
Dictionary.fn._end = function(fn) {
  if (this._ended) {
    fn(this.json);
  }
  else {
    this.on('end', function(json) {
      fn(json);
    });
  }
  return this;
};

Dictionary.fn.getData = function(page) {
  var self = this;
  this.page = page || 1;
  this.json = '';
  if (this._req && this._ended === 0) this._req.abort(); // abort if there's already one request happening
  this._ended = false;
  this._req = http.get({
    port: 80,
    host: 'www.urbandictionary.com',
    path: '/iphone/search/define?page=' + self.page + '&term=' + self.words
  }, function(res) {
    res.on('data', function(data) {
      self._ended = 0; // in progress
      self.json += data;
    }).on('end', function() {
      self.json = JSON.parse(self.json)
      self.emit('end', self.json);
      self._ended = true;
    });
  });
  return this;
};

Dictionary.fn.images = function(fn) {
  return this._end(function(json) {
    if (fn) fn(json.images);
  });
};

Dictionary.fn.total = function(fn) {
  return this._end(function(json) {
    if (fn) fn(json.total);
  });
};

Dictionary.fn.pages = function(fn) {
  return this._end(function(json) {
    if (fn) fn(json.pages);
  });
};

Dictionary.fn.more = Dictionary.fn.next = function() {
  return this.getData(++this.page);
};

Dictionary.fn.prev = function() {
  return this.getData(this.page < 2 ? 1 : --this.page);
}

Dictionary.fn.res = Dictionary.fn.results = function(fn) {
  return this._end(function(json) {
    if (fn) fn(json.list);
  });
};

Dictionary.fn.first = function(fn) {
  return this._end(function(json) {
    if (fn && json.list) fn(json.list[0]);
  });
};

module.exports = urban;
